@@ -59,10 +59,10 @@ var (
 		# Disable cert checking for the e2e cluster entry
 		kubectl config set-cluster e2e --insecure-skip-tls-verify=true
 
-		# Set custom TLS server name to use for validation for the e2e cluster entry
+		# Set the custom TLS server name to use for validation for the e2e cluster entry
 		kubectl config set-cluster e2e --tls-server-name=my-cluster-name
 
-		# Set proxy url for the e2e cluster entry
+		# Set the proxy URL for the e2e cluster entry
 		kubectl config set-cluster e2e --proxy-url=https://1.2.3.4`)
 )
 
